package edu.mermet.tp8;


import java.awt.event.*;
import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Properties;
import javax.swing.*;

import edu.mermet.tp8.fenetres.*;

/**
 *
 * @author brunomermet
 */
public class Application extends JFrame {
    private JInternalFrame conversion;
    private JInternalFrame texte;
    private JInternalFrame diaporama;
    private JInternalFrame boutons;
    private JDialog commentFaire;
    private JDialog configurationMenus;
    private Action actionAfficherConversion;
    private Action actionAfficherTexte;
    private Action actionAfficherDiaporama;
    private Action actionAfficherBoutons;
    private Action actionCommentFaire;
    private Action actionConfigurationMenus;

    private Properties preferences;

    public static final String[] menuProps = new String[]{"conversion", "saisie", "diapo", "boutons"};
    private ArrayList<JMenuItem> jMenuItems = new ArrayList<>();

    public Application() {
        super("multi-fenêtres");
        this.setContentPane(new JDesktopPane());

        retrievePreferences();

        // ****** Barre de menu ******
        JMenuBar barre = new JMenuBar();
        // ------ menu Fichier ------
        JMenu menuFichier = new JMenu("Fichier");
        menuFichier.setMnemonic(KeyEvent.VK_F);
        JMenuItem quitter = new JMenuItem("Quitter");
        quitter.addActionListener (aev -> {
            competenceIncrease(0.1, 4);
            savePreferences();
            System.exit(0);
        });
        quitter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_DOWN_MASK));
        menuFichier.add(quitter);
        barre.add(menuFichier);
        this.setJMenuBar(barre);
        // ------ menu Applications ------
        JMenu menuApplication = new JMenu("Applications");
        menuApplication.setMnemonic(KeyEvent.VK_A);
        actionAfficherConversion = new ActionAfficherConversion();
        JMenuItem itemConversion = new JMenuItem(actionAfficherConversion);
        menuApplication.add(itemConversion);
        actionAfficherTexte = new ActionAfficherTexte();
        JMenuItem itemTexte = new JMenuItem(actionAfficherTexte);
        menuApplication.add(itemTexte);
        actionAfficherDiaporama = new ActionAfficherDiaporama();
        JMenuItem itemDiaporama = new JMenuItem(actionAfficherDiaporama);
        menuApplication.add(itemDiaporama);
        actionAfficherBoutons = new ActionAfficherBoutons();
        JMenuItem itemBoutons = new JMenuItem(actionAfficherBoutons);
        menuApplication.add(itemBoutons);
        barre.add(menuApplication);

        // Ajouter dans l'ordre de menuProps
        jMenuItems.add(itemConversion);
        jMenuItems.add(itemTexte);
        jMenuItems.add(itemDiaporama);
        jMenuItems.add(itemBoutons);

        // -------- menu Aide ---------
        JMenu menuAide = new JMenu("Aide");
        menuAide.setMnemonic(KeyEvent.VK_A);
        actionCommentFaire = new ActionCommentFaire();
        JMenuItem itemCommentFaire = new JMenuItem(actionCommentFaire);
        menuAide.add(itemCommentFaire);
        actionConfigurationMenus = new ActionConfigurationMenus();
        JMenuItem itemConfigurationMenus = new JMenuItem(actionConfigurationMenus);
        menuAide.add(itemConfigurationMenus);

        barre.add(menuAide);

        // ****** Fin barre de menu ******
        
        // ****** Création des fenêtres ******
        // ------ fenêtre conversion ------
        conversion = new FenetreConversion(actionAfficherConversion);
        this.add(conversion);
        // ------ fenêtre texte ------
        texte = new FenetreTexte(actionAfficherTexte);
        this.add(texte);
        // ------ fenêtre diaporama ------
        diaporama = new FenetreDiaporama(actionAfficherDiaporama);
        this.add(diaporama);
        // ------ fenêtre boutons ------
        boutons = new FenetreBoutons(this,actionAfficherBoutons);
        this.add(boutons);
        // ------ fenêtre comment faire ------
        commentFaire = new FenetreCommentFaire(preferences);
        // ------ fenêtre config menus ------
        configurationMenus = new FenetreConfigurationMenus(preferences, this);
        // ****** Fin création fenêtres ******


        //******* Début affichage des menus *********//
        affichageMenu();
        //******* Fin affichage menus **********//

        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        WindowListener exitListener = new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                savePreferences();
                System.exit(0);
            }
        };
        addWindowListener(exitListener);
        setSize(600,300);
        this.setLocationRelativeTo(null);
        setVisible(true);

        new FenetreSuggestion(preferences);
    }

    public void affichageMenu() {
        double competence = Double.parseDouble(preferences.getProperty("competence"));
        jMenuItems.get(Arrays.asList(menuProps).indexOf("boutons")) .setVisible(!(competence < 8));
        jMenuItems.get(Arrays.asList(menuProps).indexOf("saisie")) .setVisible(!(competence < 4));
        jMenuItems.get(Arrays.asList(menuProps).indexOf("conversion")) .setVisible(!(competence < 4));
        jMenuItems.get(Arrays.asList(menuProps).indexOf("diapo")) .setVisible(!(competence < 2));
        for(String menuPref : menuProps) {
            String pref = preferences.getProperty(menuPref);
            if(pref == null) continue;
            Boolean toSet = null;
            if(pref.equals("affiche")) {
                toSet = true;
            }else if(pref.equals("cache")) {
                toSet = false;
            }
            if(toSet != null) {
                jMenuItems.get(Arrays.asList(menuProps).indexOf(menuPref)) .setVisible(toSet);
            }
        }
    }

    private void retrievePreferences() {
        try {
            String homePath = System.getProperty("user.home");
            String userName = System.getProperty("user.name");
            if(Files.isDirectory(Paths.get(homePath + "/.ihm"))) {
                if(Files.exists(Paths.get(homePath + "/.ihm/" + userName + ".xml"))) {
                    preferences = new Properties();
                    InputStream is = new FileInputStream(homePath + "/.ihm/" + userName + ".xml");
                    preferences.loadFromXML(is);
                }else {
                    initPreferences();
                }
            }else {
                Files.createDirectory(Paths.get(homePath + "/.ihm"));
                retrievePreferences();
            }
        }catch(Exception e) {
            e.printStackTrace();
        }
    }

    private void initPreferences() {
        preferences = new Properties();
        preferences.setProperty("competence", "1");
        try {
            Files.createFile(Paths.get(System.getProperty("user.home") + "/.ihm/" + System.getProperty("user.name") + ".xml"));
        }catch(IOException ioe) {
            ioe.printStackTrace();
        }
    }

    private void savePreferences() {
        try {
            OutputStream os = new FileOutputStream(System.getProperty("user.home") + "/.ihm/" + System.getProperty("user.name") + ".xml");
            preferences.storeToXML(os, "preferences");
            os.close();
        }catch(IOException ioe) {
            ioe.printStackTrace();
        }
    }

    private void competenceIncrease(double increment, double max) {
        double comp = Double.parseDouble(preferences.getProperty("competence"));
        if(comp < max) comp += increment;
        preferences.setProperty("competence", Double.toString(comp));
    }

    private class ActionAfficherBoutons extends AbstractAction {
        public ActionAfficherBoutons() {
            super("Boutons");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_B, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_B);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            boutons.setVisible(true);
            competenceIncrease(0.8, 32);
            enableBoutons(false);
        }
    }

    private class ActionAfficherDiaporama extends AbstractAction {
        public ActionAfficherDiaporama() {
            super("Diaporama");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_D);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            diaporama.setVisible(true);
            competenceIncrease(0.2, 8);
            enableDiaporama(false);
        }
    }

    private class ActionAfficherTexte extends AbstractAction {
        public ActionAfficherTexte() {
            super("Saisie de texte");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_T);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            texte.setVisible(true);
            competenceIncrease(0.4, 16);
            enableTexte(false);
        }
    }
    
    public class ActionAfficherConversion extends AbstractAction {
        public ActionAfficherConversion() {
            super("Conversion Celsius/Farenheit");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_C);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            conversion.setVisible(true);
            competenceIncrease(0.4, 16);
            enableConversion(false);
        }
    }

    private class ActionCommentFaire extends AbstractAction {
        public ActionCommentFaire() {
            super("Comment faire ?");
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_C);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            commentFaire.setVisible(true);
        }
    }

    private class ActionConfigurationMenus extends AbstractAction {
        public ActionConfigurationMenus() {
            super("Configuration des menus");
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_C);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            configurationMenus.setVisible(true);
        }
    }

    public void enableConversion(boolean b) {
        actionAfficherConversion.setEnabled(b);
    }

    public void enableTexte(boolean b) {
        actionAfficherTexte.setEnabled(b);
    }

    public void enableDiaporama(boolean b) {
        actionAfficherDiaporama.setEnabled(b);
    }

    public void enableBoutons(boolean b) {
        actionAfficherBoutons.setEnabled(b);
    }

    public Action getActionAfficherConversion() {
        return actionAfficherConversion;
    }

    public Action getActionAfficherTexte() {
        return actionAfficherTexte;
    }

    public Action getActionAfficherDiaporama() {
        return actionAfficherDiaporama;
    }
    public static void main(String[] args) {
        SwingUtilities.invokeLater(Application::new);
    }

    
}
